(function () {
	'use strict';

	angular.module('ngApp').requires.push('ionic-audio');

	angular
		.module('ngApp')
		.controller('MusicCtrl', [
			'$scope',
			'$rootScope',
			'$ionicModal',
			'$http',
			'$ionicScrollDelegate',
			'$sce',
			'MediaManager',
			'localStorage',
			musicController
		]);
		
	function musicController($scope, $rootScope, $ionicModal, $http, $ionicScrollDelegate, $sce, MediaManager, localStorage) {
		
		var REPEAT_STATUS = {
			NO_REPEAT: 0,
			REPEAT_ONE: 1,
			REPEAT_ALL: 2
		};

		$scope.track = {};

		$scope.repeatStatus = localStorage.all('musicRepeatStatus', 0);
		$scope.shuffleStatus = localStorage.all('musicShuffleStatus', false);

		var musicPlayed = [];

		function playMusic(id) {
			if ($scope.musics[id]) {
				$scope.track = $scope.musics[id];
				$scope.track.index = id;
				$scope.togglePlayback = !$scope.togglePlayback;
				$scope.track.hasNext = hasNextButton();
				$scope.track.hasPrevious = hasPreviousButton();
				$scope.track.next = $scope.next;
				$scope.track.previous = $scope.previous;
				addPlayedMusic(id);
			}
		}

		$scope.forward = function () {
			MediaManager.seekTo($scope.track.progress + 15);
		};

		$scope.backward = function () {
			MediaManager.seekTo($scope.track.progress - 15);
		};

		$scope.$watch('track.progress', function (newValue, oldValue) {
			if (newValue >= ($scope.track.duration - 2) && newValue >= 1) {
				if ($scope.repeatStatus == REPEAT_STATUS.REPEAT_ONE) {
					$scope.repeatOne();
				} else {
					$scope.next();
				}
			}
		});

		$scope.next = function () {
			var id = $scope.track.index + 1;
			if ($scope.shuffleStatus) {
				id = getSuffleMusicId();
			}
			if ($scope.musics[id]) {
				playMusic(id);
			} else if ($scope.repeatStatus == REPEAT_STATUS.REPEAT_ALL) {
				$scope.track.index = -1;
				$scope.next();
			}
		};

		$scope.previous = function () {
			var id = $scope.track.index - 1;
			if ($scope.musics[id]) {
				playMusic(id);
			} else if ($scope.repeatStatus == REPEAT_STATUS.REPEAT_ALL) {
				playMusic($scope.musics.length - 1);
			}
		};

		$scope.repeat = function () {
			if (++$scope.repeatStatus > 2) {
				$scope.repeatStatus = 0;
			}
			localStorage.save($scope.repeatStatus, 'musicRepeatStatus');
		};

		$scope.repeatOne = function () {
			MediaManager.seekTo(0);
		};

		$scope.shuffle = function () {
			$scope.shuffleStatus = !$scope.shuffleStatus;
			localStorage.save($scope.shuffleStatus, 'musicShuffleStatus');
		};

		$scope.download = function (name) {
			if (isDevice && ionic.Platform.isAndroid()) {
				var permissions = cordova.plugins.permissions;
				var url = encodeURI(decodeURIComponent($scope.track.url));
				var permissionerror = function () {
					showWarning('برای ذخیره فایل نیاز به اجازه دسترسی فایل هست ، لطفا داخل تنظیمات اپلیکیشن اجازه دسترسی به فایل را روشن کنید');
				};

				permissions.requestPermission(
					permissions.WRITE_EXTERNAL_STORAGE,
					function (status) {
						if (status.hasPermission) {
							cordova.plugins.DownloadManager.download(
								url,
								function (data) {
									showSuccess('در حال دریافت ...');
								},
								function (error) {
									showWarning('خطا در دریافت');
								}
							);
						} else {
							permissionerror();
						}
					},
					permissionerror
				);
			} else {
				window.open($scope.track.url, '_system');
			}
		};

		// Wait until the sl var is init
		$scope.$watch('sl', function () {
			if (typeof $scope.sl != 'undefined') {
				$scope.getMusics(false);
			}
		});

		$scope.getMusics = function (refresh) {
			if (isOnline()) {
				if (!refresh) {
					$scope.loading = true;
				}
				$http({
						method: 'GET',
						noLoading: true,
						url: url + '/api/v1/widget/route/music/getMusics/' + $scope.sl + '?length=10'
					})
					.success(function (data) {
						$scope.$broadcast('scroll.refreshComplete');
						$scope.storage = '';
						$scope.musics = data.musics;
						$scope.start_content = $sce.trustAsHtml(data.start_content);
						if ($scope.musics.length == 10) {
							$scope.noMoreItemsAvailable = false;
						}
						$scope.loading = false;
					})
					.error(function (error) {
						$scope.$broadcast('scroll.refreshComplete');
						$scope.storage = 'خطا در اتصال به سرور';
						$scope.loading = false;
						checkYourInternet();
					});
			} else {
				$scope.loading = false;
				showToast("اتصال اینترنت خود را بررسی کنید");
			}
		};

		$scope.doRefresh = function () {
			$scope.getMusics(true);
		};

		$scope.loadMore = function () {
			if (!$scope.noMoreItemsAvailable) {
				$scope.noMoreItemsAvailable = true;
				if (isOnline() && typeof $scope.musics == "object") {
					var start = $scope.musics.length;
					$scope.infiniteScrollLoading = true;
					$http({
							method: 'GET',
							noLoading: true,
							url: url + '/api/v1/widget/route/music/getMusics/' + $scope.sl + '/' + start + '?length=10'
						})
						.success(function (data) {
							if (data.musics.length == 0 || data.found == 0) {
								$scope.noMoreItemsAvailable = true;
							} else {
								for (var i = 0; i < data.musics.length; i++) {
									$scope.musics.push(data.musics[i]);
								}
							}
							if (data.musics.length < 10) {
								$scope.noMoreItemsAvailable = true;
							} else {
								$scope.noMoreItemsAvailable = false;
							}
							$scope.infiniteScrollLoading = false;
							$scope.$broadcast('scroll.infiniteScrollComplete');
						})
						.error(function () {
							console.error('ERR', err.status);
							$scope.infiniteScrollLoading = false;
							$scope.$broadcast('scroll.infiniteScrollComplete');
						});
				}
			}
		};

		$ionicModal.fromTemplateUrl('music-modal.html', {
			scope: $scope,
			animation: 'slide-in-up',
			name: 'musicModal',
			hardwareBackButtonClose: true
		}).then(function (modal2) {
			$scope.musicmodal = modal2;
		});

		$scope.openMusic = function (index) {
			if ($scope.track.index != index) {
				playMusic(index);
			}
			$rootScope.openModal = $scope.musicmodal;
			$scope.musicmodal.show();
		};

		$ionicModal.fromTemplateUrl('content-modal2.html', {
			scope: $scope,
			animation: 'slide-in-up',
			hardwareBackButtonClose: true
		}).then(function (modal) {
			$scope.contentmodal = modal;
		});

		$scope.openContent = function () {
			if ($scope.track.content != '') {
				if (typeof $scope.track.content == 'string') {
					$scope.track.content = $sce.trustAsHtml($scope.track.content);
				}
				$rootScope.openModal = $scope.contentmodal;
				$scope.contentmodal.show();
				$ionicScrollDelegate.$getByHandle('contentmodal').scrollTop();
			}
		};

		$scope.share = function () {
			var message = $scope.track.name + "\n" + $scope.track.artist;
			var link = $scope.track.url;
			window.plugins.socialsharing.share(message, null, null, link);
		}

		$scope.openNote = function () {
			$ionicModal.fromTemplateUrl('content-modal.html', {
				scope: $scope,
				animation: 'slide-in-up',
				hardwareBackButtonClose: true
			}).then(function (modal) {
				$rootScope.openModal = modal;
				$rootScope.openModal.show();
				$ionicScrollDelegate.$getByHandle('modal').scrollTop();
			});
		};

		$scope.$on('modal.hidden', function (event, modal) {
			if (modal.name == 'musicModal') {
				MediaManager.destroy();
			}
		});
		
		function addPlayedMusic(id) {
			var index = musicPlayed.indexOf(id);
			if (index !== -1) {
				musicPlayed.splice(index, 1);
			}
			musicPlayed.push(id);
			if ($scope.repeatStatus == REPEAT_STATUS.REPEAT_ALL && musicPlayed.length == $scope.musics.length) {
				musicPlayed = [];
			}
		}

		function hasNextButton() {
			if ($scope.repeatStatus == REPEAT_STATUS.REPEAT_ALL) {
				return true;
			}
			if ($scope.shuffleStatus) {
				return musicPlayed.length < $scope.musics.length;
			}
			return (typeof $scope.musics[$scope.track.index + 1] !== "undefined");
		}

		function hasPreviousButton() {
			if ($scope.repeatStatus == REPEAT_STATUS.REPEAT_ALL) {
				return true;
			}
			// if ($scope.shuffleStatus) {
			// 	return musicPlayed.length > 1;
			// }
			return (typeof $scope.musics[$scope.track.index - 1] !== "undefined");
		}

		function getSuffleMusicId() {
			var options = [];
			for (var i = 0; i < $scope.musics.length; i++) {
				if (musicPlayed.indexOf(i) == -1) {
					options.push(i);
				}
			}
			if (options.length) {
				var id = rand(options.length);
				return options[id];
			}
			return -1;
		}

		function rand(max) {
			return Math.floor(Math.random() * max);
		}
	}
})();